/*
*   Test Problem 2 - - ** Np tasks = 12 **
*
*   Program Name -- EUIprob2.c --
*
*   Modified for EUI testing by Ron Mraz, July, 1992
*   Adapted from FORTRAN to C by Joe Riordan, June, 1993
*
*   Problems to test Parallel and Vector Languages
*   CSD-TR-1016, Computer Science, Purdue University
*   John R. Rice and Jin Jing, October 1, 1990
*
*   This program evaluates e* for a series of cases.
*   This is essentially the sum of products of (1 - exp() ).
*   To examine parallel execution, a producer consumer senario
*   is set up where the product results are sent to a waiting node
*   for summing. There are 6 pairs of nodes working independently.
*   Sends and receives are fine grain in nature.
*
*   This program requires the following call(s)
*      MPI_Init, MPI_Comm_rank, MPI_Comm_size, MPI_Send, MPI_Recv, MPI_Finalize
*
*   LAST MPL VERSION REVISION: 9/15/93 for latest API changes. BMB
*                  5/26/94 set exit value.  RYL
*   CONVERTED TO MPI: 11/12/94 by Xianneng Shen
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"


#define NDIM 320
#define MDIM 360
#define KASES 6

MPI_Status status;
main (int argc, char **argv) 
{

    float	TEMP[NDIM][MDIM], PROD[MDIM], TESTAR[KASES], ESTAR,
		T1, sdata, rdata;
    int		K, KK, slen, sdest, stype, rlen, rdest, rtype, nbytes,
		nworkers, jwhoami, I, J, 
                nbuf[4],
        	N[KASES] = {10,20,40,80,160,320},
        	M[KASES] = {15,30,60,90,180,360};

/*
*    Initialize environment
*/
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &jwhoami);
    MPI_Comm_size(MPI_COMM_WORLD, &nworkers);

    if (nworkers != 12 ) {
        printf("Usage NP = 12, You selected %d. This is node %d\n",nworkers,
                jwhoami);
        exit(0);
    }

    printf("Environment allocated %d and we are node %d\n",nworkers,jwhoami);

/*
*    divide the problem into a producer/consumer pair for each case.
*    a node with an even number is a producer, odd is a consumer
*/

    if ((jwhoami % 2) == 0) {
/*  we are a producer, our consumer is node (jwhoami +1) */

/*  K is our case study */
        K = (jwhoami/2);
        printf("We are executing case %d\n",K+1);
        for (I = 0; I < N[K]; I++) {
             for (J = 0; J < M[K]; J++) {
                  T1 = ((float) (-abs(I-J)));
                  TEMP[I][J] = 1+((float) (exp(T1)));
             }
             PROD[I] = 1;
             for (J = 0; J < M[K]; J++)
                 PROD[I] = PROD[I]*TEMP[I][J];
             sdata = PROD[I];
             slen = 4;
             sdest = jwhoami + 1;
             stype = 0;
             printf("bsend buf %8.4f, len %d, dest %d, type %d\n",sdata,
                 slen,sdest,stype);
             MPI_Send(&sdata, 1, MPI_FLOAT, sdest, stype, MPI_COMM_WORLD);
              
        }
    }
/*     end of producer code */

    if ((jwhoami % 2) == 1) {
/*      this is consumer code, our producer is node (jwhoami - 1)
*       therefore the case we are working on is (jwhoami - 1)
*/
        KK = (jwhoami/2);
        printf("We are executing case %d\n",KK+1);
        ESTAR = 0;
        printf("%d\n",N[KK]);
        for (I = 0; I < N[KK]; I++) {
            rlen = 4;
            rdest = jwhoami - 1;

            nbytes = rlen;
            MPI_Recv(&rdata, 1, MPI_FLOAT, rdest, rtype, MPI_COMM_WORLD, &status);
            printf("brecv buf %8.4f, dest %d\n",rdata, rdest);
            PROD[I] = rdata;
            ESTAR = ESTAR+PROD[I];
        }
        TESTAR[KK] = ESTAR;

        printf("Problem 2 with N = %d, M = %d\n",N[KK],M[KK]);
        printf("Node %d give ESTAR = %8.4f\n",jwhoami,TESTAR[KK]);
    }
    MPI_Finalize();
}
