/*
*   Test Problem 3 - - ** Np tasks = 8 **
*
*   Program Name -- EUIprob3.c --
*
*   Developed for EUI testing by Ron Mraz, July, 1992
*   Adapted from FORTRAN to C by Joe Riordan, June, 1993
*
*   Problems to test Parallel and Vector Languages
*   CSD-TR-1016, Computer Science, Purdue University
*   John R. Rice and Jin Jing, October 1, 1990
*
*
*   This is essentially the sum of products of an initialized matrix.
*   To examine parallel execution, a producer consumer senario
*   is set up where the product results are sent to a waiting node
*   for summing. This differs from problem 2 in that all values are
*   sent in a single message and truncation is relied upon for message
*   passing efficiency on the receiving side.
*
*   This program requires the following call(s)
*
*     MPI_Init, MPI_Comm_rank, MPI_Comm_size, MPI_Send, MPI_Recv, MPI_Finalize
*
*   LAST MPL VERSION REVISION: 9/15/93 for latest API changes.  BMB
*                  5/24/94 set exit value. RYL
*   CONVERTED TO MPI: 11/12/94 by Xianneng Shen
*
*/

#include <stdlib.h>
#include <stdio.h>
#include "mpi.h"
#define NDIM 270
#define MDIM 625
#define KASES 4
MPI_Status status;
main(int argc, char **argv) 
{

    float	A[NDIM][MDIM], P[NDIM], TS[KASES], S,
		sdata[MDIM], rdata[MDIM];
    int		nrules[2], ntype, nworkers, whoami,
		slen, sdest, stype, rlen, rdest, rtype, nbytes,
		I, J, K, NN, MM,
		N[KASES] = {10,30,90,270},
		M[KASES] = {5,25,125,625};
/*
*       Initialize environment
*/
    nrules[0] = 1;
    nrules[1] = 1;
    ntype = 1;
    for (I=0; I < NDIM; I++) {
         for (J=0; J < MDIM; J++) {
              A[I][J] = 1.0001;
         }
    }
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &whoami);
    MPI_Comm_size(MPI_COMM_WORLD, &nworkers);
    if( whoami > 8 ) {
        printf("Usage NP = 8, You selected %d. This is node %d\n",nworkers,
            whoami);
        exit(0);
    }

    printf("Environment allocated %d and we are node %d\n",nworkers,
        whoami);
/*
*    divide the problem into a producer/consumer pair for each case.
*    a node with an even number is a producer, odd is a consumer
*/

    if ((whoami % 2) == 0) {
/*  we are a producer, our consumer is node (whoami +1) */

        K = (whoami/2);
        printf("We are executing case %d\n",K+1);
        NN = N[K];
        MM = M[K];
        for (I=0; I < NN; I++) {
            P[I] = 1;
            for (J=0; J < MM; J++) {
                P[I] = (P[I] * A[I][J]);
            }
        }

/*  Send the complete data set over to the consumer */
        slen = 4 * NDIM;
        sdest = whoami + 1;
        stype = 0;
        printf("bsend buf %8.4f, len %d, dest %d, type %d\n",P[NN-1],
            slen,sdest,stype);
        MPI_Send(&P, NDIM, MPI_FLOAT, sdest, stype, MPI_COMM_WORLD);
    }
/*  end of producer code */

    if ((whoami % 2) == 1) {
/*      this is consumer code, our producer is node (whoami - 1)
*       therefore the case we are working on is (whoami - 1)
*/
        K = (whoami/2); 
        printf("We are executing case %d\n",K+1);

        NN = N[K];
        MM = M[K];
        S = 0;
/*  len is size of matrix */
        rlen = 4 * NDIM;
        rdest = whoami - 1;
        nbytes = 4*NN;
        MPI_Recv(&rdata, NDIM, MPI_FLOAT, rdest, 0, MPI_COMM_WORLD, &status);
        printf("brecv buf %8.4f, len %d, dest %d\n",rdata[NN-1],
                 rlen,rdest);
        for (I=0; I < NN; I++) {
            P[I] = rdata[I];
            S = S+P[I];
        }
        TS[K] = S;

        printf("Problem 3 with N = %d, M = %d\n",N[K],M[K]);
        printf("Node %d gives S = %8.4f\n",whoami,TS[K]);
    }
    MPI_Finalize();
}
