/*
*   Test Problem 5 - - ** Np tasks = 6 **
*
*   Program Name -- EUIprob5.c --
*
*   Modified for EUI testing by Ron Mraz, July, 1992
*   Adapted from FORTRAN to C by Joe Riordan, June, 1993
*
*   This test program tabulates statistics of a set of test scores.
*   A separate processor is used to calculate one of 5 databases.
*   The program test the environments WAIT synchronization.
*
*   Problems to test Parallel and Vector Languages
*   CSD-TR-1016, Computer Science, Purdue University
*   John R. Rice and Jin Jing, October 1, 1990
*
*   This program in parallel form requires the following call(s)
* 
*     MPI_Init, MPI_Comm_rank, MPI_Comm_size, MPI_Isend, MPI_Irecv, MPI_Wait,
*     MPI_Finalize
*
*   LAST MPL VERSION REVISION: 9/15/93 for latest API changes.  BMB
*                  5/26/94 set exit value.  RYL
*                  11/7/94 correct use of msgid,
*                          correction to XMAX function declaration. RYL
*   CONVERTED TO MPI: 11/12/94 by Xianneng Shen
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <mpproto.h>
#include "mpi.h"
#define NTDIM 50
#define NSDIM 1000
#define KASES 5
#define FALSE 0
#define TRUE  1

MPI_Status status;
MPI_Request request;
main(int argc, char **argv) 
{

    float	SCORES[NTDIM][NSDIM], TOP[NSDIM], TAVER[KASES], 
        	TLOWABO[KASES], AVER, LOWABO, SUM,
		XMAX(float [][NSDIM], int, int);
    int		nbytes, src, dest, type, msgid,
		I, J, K, NT, NS, NABOVE,
		nworkers, kwhoami, ntype,
		N[KASES] = {5,20,30,40,50},
		M[KASES] = {12,30,200,400,1000},
		ABOVE[NTDIM][NSDIM], TGENIUS[KASES], G[NSDIM], GENIUS;
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &kwhoami);
    MPI_Comm_size(MPI_COMM_WORLD, &nworkers);
    if( nworkers != 6 ) {
        printf("Usage NP = 6, You selected %d. This is node %d\n",nworkers,
            kwhoami);
        exit(0);
    }

    printf("Environment allocated %d and we are node %d\n",nworkers,kwhoami);

/*
*    Divide the problem into 5 independent pieces.
*    Each piece sends their results to node zero for printout
*/

    if (kwhoami == 0) {

/*  Receive all messages
*
*     messages are either type 1, 2, or 3
*        Type 1 messages are for AVER, float
*        Type 2 messages are for GENIUS, int
*        Type 3 messages are for LOWABO, float
*/
        for (K = 1; K <= KASES; ++K) {
            src = K;
            nbytes = 4;
            ntype = 1;
         MPI_Irecv(&AVER, 1, MPI_FLOAT, src, ntype, MPI_COMM_WORLD, &request);
         MPI_Wait(&request, &status); 
            TAVER[K-1] = AVER;

            nbytes = 1;
            ntype = 2;
          MPI_Irecv(&GENIUS, 1, MPI_INT, src, ntype, MPI_COMM_WORLD, &request);
          MPI_Wait(&request, &status);
            TGENIUS[K-1] = GENIUS;

            nbytes = 4;
            ntype = 3;
         MPI_Irecv(&LOWABO, 1, MPI_INT, src, ntype, MPI_COMM_WORLD, &request);
         MPI_Wait(&request, &status);
            TLOWABO[K-1] = LOWABO;
            printf("Problem 5, Case %d, with NT = %d, NS = %d\n", K,
                N[K-1], M[K-1]);
            printf("gives AVERAGE = %8.4f, GENIUS = %d\n", TAVER[K-1],
                TGENIUS[K-1]);
            printf("and low-above = %8.4f\n",TLOWABO[K-1]);
        }
    }

    else {

/*  This is a compute node */

        K = kwhoami;
        dest = 0;
        for (I=1; I <= NTDIM; ++I)
            for (J=1; J <= NSDIM; ++J)
                SCORES[I-1][J-1] = 60. + 40. * ((float) (sin(I*J*63.21)));
        NABOVE = 0;
        LOWABO = 9.9E10;
        NT = N[K-1];
        NS = M[K-1];
        for (I=0; I < NT; ++I)
            TOP[I] = XMAX(SCORES,NSDIM,I);
        SUM = 0.0;
        for (I=0; I < NT; ++I)
            for (J=0; J < NS; ++J)
                SUM = SUM + SCORES[I][J];
        AVER = SUM/(NS*NT);
        nbytes = 4;
        type = 1;
        MPI_Isend(&AVER, 1, MPI_FLOAT, dest, type, MPI_COMM_WORLD, &request);
        

/*    Send average to node zero */

        for (I = 0; I < NT; ++I)
            for (J = 0; J < NS; ++J) {

/*    Increase Scores Above Average */

                if (SCORES[I][J] > AVER) 
                    SCORES[I][J] = 1.1*SCORES[I][J];

/*    Set Switch for Scores Above Average */

                ABOVE[I][J] = FALSE;
                if (SCORES[I][J] > AVER) {
                    NABOVE = NABOVE+1;
                    ABOVE[I][J] = TRUE;
                }
            }
        printf("NABOVE = %d\n",NABOVE);

/*    Set Switch for Scores Above Average */

        LOWABO = 100.0;
        for (I = 0; I < NT; ++I)
            for (J = 0; J < NS; ++J)
                if (ABOVE[I][J] == TRUE)
                    if (SCORES[I][J] < LOWABO)
                        LOWABO = SCORES[I][J];
      MPI_Wait(&request, &status);

/*    Send LOWABO to node zero */

        type = 3;
       MPI_Isend(&LOWABO, 1, MPI_INT, dest, type, MPI_COMM_WORLD, &request);

        for (I = 0; I < NT; ++I) {
            G[I] = TRUE;
            for (J = 0; J < NS; ++J)
                if (ABOVE[I][J] != TRUE)
                    G[I] = FALSE;
        }

/*    GENIUS = ANY(G) */

        GENIUS = FALSE;
        for (I = 0; I < NT; ++I)
            if (G[I] == TRUE)
                GENIUS = TRUE;
       MPI_Wait(&request, &status);

/*    Send GENIUS to node zero */

        type = 2;
        nbytes = 1;
        MPI_Isend(&GENIUS, 1, MPI_INT, dest, type, MPI_COMM_WORLD, &request);
        MPI_Wait(&request, &status);

    }
    MPI_Finalize();
}

/* MAX Function to handle find the Maximum Value in a row of an array */
float XMAX(float A[][NSDIM], int M, int I)
{
    float	XXMAX;
    int 	JJ;

    XXMAX = 1.0E-10;
    for (JJ = 0; JJ < M; ++JJ)
        if (A[I][JJ] > XXMAX)
            XXMAX =  A[I][JJ];
    return XXMAX;
}
