/*
*   Test Problem 6 - - ** Np tasks = 4 **
*
*   Program Name -- EUIprob6.c --
*
*   Modified for EUI testing by Ron Mraz, July, 1992
*   Adapted from FORTRAN to C by Joe Riordan, June, 1993
*
*   This test program solves the tridiagnonal system Tx = y by the
*   special, vector oriented algorithm of [Jorden, 1979]. The T matrix
*   is represented by L, D, and U, as its lower, main diagnonal and
*   upper diagonal, respectively.
*
*   Problems to test Parallel and Vector Languages
*   CSD-TR-1016, Computer Science, Purdue University
*   John R. Rice and Jin Jing, October 1, 1990
*
*   This program in parallel form requires the following call(s)
*
*     MPI_Init, MPI_Comm_rank, MPI_Comm_size, MPI_Sendrecv, MPI_Finalize
*
*   LAST MPL VERSION REVISION: 9/15/93 for latest API changes.  BMB
*                  5/27/94 set exit value. RYL
*   CONVERTED TO MPI: 11/12/94 by Xianneng Shen
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#define MAX 16484
#define KASES 4

MPI_Status status;
main(int argc, char **argv) 
{

    float	L[MAX],D[MAX],U[MAX],X[MAX],Y[MAX],T[MAX],
		SUMX, SOLUT[KASES];

    int		NI[KASES] = {128,1024,4096,16384},
		slen,dest,type,rlen,src,nbytes,nworkers,kwhoami,
		I, II, N, M, Ndest, Mdest, NMAX,
		LIMIT, SLIMIT, K, temp, LBUF, NBUF, MBUF;
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &kwhoami);
    MPI_Comm_size(MPI_COMM_WORLD, &nworkers);
    if( nworkers != 4 ) {
        printf("Usage NP = 4, You selected %d. This is node %d\n",nworkers,
            kwhoami);
        exit(0);
    }

    printf("Environment allocated %d and we are node %d\n",nworkers,kwhoami);

/*  divide the problem into 4 independent pieces.
*
*
*        Assign a case for each processor
*/

    slen = 4;
    rlen = 4;
    nbytes = 4;
    LIMIT = 0;
    if (kwhoami == 0) {
        dest = 1;
        type = 10;
    }
    if (kwhoami == 1) {
        dest = 0;
        type = 10;
    }
    if (kwhoami == 2) {
        dest = 3;
        type = 20;
    }
    if (kwhoami == 3) {
        dest = 2;
        type = 20;
    }

/*  calculate the limit for the destination machine */

    Mdest = dest + 1;
    Ndest = NI[dest];
    SLIMIT = ((int) (1.44269504*(log(Ndest))+0.01));
    printf("dest %d, send limit %d\n",dest,SLIMIT);

    src = dest;

    MPI_Sendrecv(&SLIMIT, 1, MPI_INT, dest, type, &LBUF, 1, MPI_INT, src,
         type, MPI_COMM_WORLD, &status);
    LIMIT = LBUF;
    type = type + 1;
    MPI_Sendrecv(&Ndest, 1, MPI_INT, dest, type, &NBUF, 1, MPI_INT,
          src, type, MPI_COMM_WORLD, &status);

    N = NBUF;
    type = type + 1;
    MPI_Sendrecv(&Mdest, 1, MPI_INT, dest, type, &MBUF, 1, MPI_INT, 
        src, type, MPI_COMM_WORLD, &status);

    M = MBUF;
    printf("Node %d Received %d From %d\n",kwhoami,LIMIT,src);
    printf("Node %d Received %d From %d\n",kwhoami,N,src);
    printf("Node %d Received %d From %d\n",kwhoami,M,src);

    K = 1;
/*
*    Processor exchanges are as follows 0 <--> 1
*                                       2 <--> 3
*
*
*           Create Matrix for all cases..
*/
    NMAX = NI[KASES-1];
    for (I = 1; I <= NMAX; I++) {
        L[I-1] = 0.88-0.1*((float) sin(I*12.36));
        D[I-1] = 1.0+0.01*((float) cos(I*8.11));
        U[I-1] = 0.75+0.2*((float) sin(I*36.12+3.2));
        Y[I-1] = 1.0;
    }

    for (II = 0; II < LIMIT; II++) {
        for (I = 0; I < N; I++) {
            L[I] = L[I]/D[I];
            U[I] = U[I]/D[I];
            Y[I] = Y[I]/D[I];
        }
/*
*     T is a temporary array
*   Compute and assign to D, Compute Y
*/
        for (I = 0; I < K; I++) {
            D[I] = 1.0-U[I]*L[I+K];
            T[I] = Y[I]-U[I]*L[I+K];
        }
        for (I = K; I < N-K; I++) {
            D[I] = 1.0-L[I]*U[I-K]-U[I]*L[I+K];
            T[I] = Y[I]-L[I]*Y[I-K]-U[I]*L[I+K];
        }
        for (I = N-K; I < N; I++) {
            D[I] = 1.0-L[I]*U[I-K];
            T[I] = Y[I]-L[I]*Y[I-K];
        }

/*      Assign to Y, Compute L */

        for (I = 0; I < K; I++) {
           Y[I] = T[I];
           T[I] = 0.0;
        }
        for (I = K; I < N; I++) {
           Y[I] = T[I];
           T[I] = -L[I]*L[I-K];
        }

/*      Assign to L, Compute U */

        for (I = 0; I < N-K; I++) {
            L[I] = T[I];
            T[I] = U[I]+U[I+K];
        }
        for (I = N-K; I < N; I++) {
            L[I] = T[I];
            T[I] = 0.0;
        }

/*      Assign to U */

        for (I = 0; I < N; I++)
            U[I] = T[I];
        K = 2*K;
    }

/*      end of main loop */

    for (I = 0; I < N; I++)
        X[I] = Y[I]/D[I];
    SUMX = 0;
    for (I = 0; I < N; I++)
        SUMX = SUMX+X[I];
    SOLUT[M-1] = SUMX;

/*       M = kwhoami +1 */

    printf("Problem 6 with N = %d\n",NI[M-1]);
    printf("gives solution   = %8.4f\n",SOLUT[M-1]);
    MPI_Finalize();
}
