/* ----------------------------------------------------------------
 * ring.c:
 * DESCRIPTION:
 * This MPI program sends a series of messages in a ring.
 * Master program queries user for message to be sent, waits for
 * message to complete circuit before repeating process.  A message
 * value of -1 indicates this is the last message to be sent.
 *
 * AUTHOR: Roslyn Leibensperger
 * REVISED: 2/25/94 RYL
 * Converted from MPL to MPI by Xianneng Shen     11/20/94
 * ----------------------------------------------------------------
 */

#include <stdio.h>
#define MASTER 0
#include "mpi.h"

main(int argc, char **argv)
     { 
     int ntasks, taskid, right, left, inmsg, outmsg, tag;
     MPI_Request  request;
     MPI_Status   status;
     char tchar[8];
     outmsg = 0;

/* learn number of tasks in partition and task ID */
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &taskid );
     MPI_Comm_size(MPI_COMM_WORLD, &ntasks );
/* compute source and destination for messages */
     if (taskid == ntasks-1) right = 0;
     else right = taskid + 1;
     if (taskid == 0) left = ntasks - 1;
     else left = taskid - 1;
     tag = 1;
     while (outmsg != -1)
       {
/* master queries user for message, sends it to the right,
 * then waits for its return
 */
	if (taskid == MASTER) 
	  {
	  printf("Enter integer value to be passed along ring\n");
	  printf("A value of -1 ends the program\n");
	  scanf("%s", tchar);
	  outmsg = atoi (tchar);
          inmsg = 0;
 MPI_Isend(&outmsg,1,MPI_INT,right,tag,MPI_COMM_WORLD,&request);
         MPI_Wait(&request, &status);
	  MPI_Recv(&inmsg, 1, MPI_INT, left, tag, MPI_COMM_WORLD, &status);
	  printf ("MASTER received message %d, content is %d\n", tag, inmsg);
	  }

/* worker reads message, passes it on
 * the use of mp_send/mp_wait here is equivalent to mp_bsend
 */ 
	else
	  {
	  MPI_Recv(&inmsg, 1, MPI_INT, left, tag, MPI_COMM_WORLD, &status);
	  outmsg = inmsg;
	 MPI_Isend (&outmsg, 1, MPI_INT, right, tag, MPI_COMM_WORLD, &request);
	 MPI_Wait (&request, &status);
	  printf ("%d processed message %d, content is %d\n", taskid, 
                   tag, outmsg);
	  }
	tag++;
      }
     return 0;
   }


