/*	This is the code for the CPS 615 Spring Final project for
	"Ecological Computation".
   Authors:	Steve Alexander
   		Marjorie Quant

*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "mpi.h"

#define GENERATIONS 100         /*number of generations*/
#define MUT_ENVIRONMENT 10     /*environmental mutation rate*/
#define X_OVER 50            /*crossover rate*/
#define MUT_IND 20          /*individual mutation rate*/
#define NVARS 10            /*number of genes*/
#define MAX_POP 10000       /*largest population size*/
#define TRUE 1 
#define FALSE 0
#define START_POP 50	/*for print function */

typedef struct		/*gtype,a member of the population*/
{
   int gene[NVARS];	/*a string of variables*/
}  gtype;

gtype population[MAX_POP];		/*population*/
gtype new_population[MAX_POP]; /*new populaiton, 
                               minus bad individuals*/
gtype environment;

/* gtype migrate[NUM_MIGRATE]; */

int popsize = 50;
float avg_pop_fit = 0.0;
int num_survivors;

int randval(int low, int high) /*random number generator*/
{
  int val;		/* return 0-9 */
  val = (int)(((float)(rand()%1000)/1000.0) * (float)(high - low) + (float)low);
  return(val);
} /* end randval */

void clear(gtype *one)  /*clears a population's genes*/
{
  int i, j;
  for (i=0; i<NVARS; i++)
   	one->gene[i] = 0;
 }	/* end of clear */

void init(gtype *one)  /*initializes a population*/
{
  int i, j;

  for (i=0; i < NVARS; i++)
    one->gene[i] = randval(0, 10);

}	/* end of init */

/*************************************************************
			EVALUATE AND SELECT FUNCTION
This function evaluates the fitness of each individual and the
collective population.  Each trait of each individual is compared
with the environment.  If there is more than an 80% differnce
in any gene, the individual is selected to die.

If the average differnce is more than 50%, that individual will
be selected to die.

Only individuals not selected to die will be copied into the
new population.
*************************************************************/
void evaluate()
{
  int i, j, k;
  float ind_fit = 0.0;
  int gene_fit[NVARS];
  float avg_ind_fit = 0.0;
  float pop_fit = 0.0;
 
  int bad;

  for(i = 0; i < popsize; i++)
      clear(&(new_population[i]));

  num_survivors = 0;
  for (j=0; j < popsize; j++)
  {
    avg_ind_fit = 0.0;
    ind_fit = 0.0;

    for (i=0; i < NVARS; i++)
    {
      gene_fit[i] = (abs)(population[j].gene[i] - environment.gene[i]);
      ind_fit += (float) gene_fit[i];
    }

    avg_ind_fit = ind_fit/(float) NVARS;
    pop_fit += avg_ind_fit;
    bad = FALSE;

    for (i=0; i<NVARS; i++)
    {
      if(gene_fit[i] > 8 || avg_ind_fit > 5.0)
      {
        bad = TRUE;
      /*if gene fitness is less than 8 AND the average individual
      fitness is greater than 5, keep the individual, else don't
      copy the individual to the new population*/
      }
    }
    if(bad == FALSE)
    {

      for(k = 0; k < NVARS; k++)
      {
	/*-----*/
	if (num_survivors > MAX_POP)
	  break;
	/*-----*/
        new_population[num_survivors].gene[k] = population[j].gene[k];
        num_survivors++;   /*leaves no blank spots in new array */
      }
    }
    /*-----*/
    if (num_survivors > MAX_POP)
      break;
    /*-----*/
  }

  popsize = num_survivors;

  if(num_survivors != 0)
    avg_pop_fit = pop_fit/(float) popsize;
  else
    avg_pop_fit = 0.0;

}	/*end of evaluate */

/*******************************************************************
	MUTATION FOR ASEXUAL AND SEXUAL REPRODUCTION
*******************************************************************/
void mutate()
{
  int i, j, k;

  for (i=0; i<popsize; i++)
    {
     for (j=0; j<NVARS; j++)
      {
       k = randval(0, 100);
       if (k < MUT_IND)
       {
         if (k < MUT_IND/2)
           new_population[i].gene[j] = new_population[i].gene[j] + 1;
         else
           new_population[i].gene[j] = new_population[i].gene[j] - 1;
       }
      }
     }
}	/*end of mutate */

/********************************************************************
	SEX_REPRODUCE FUNCTION --Creates 2 sexual individuals
      for every existing sexual individual before mutation or X-over
********************************************************************/
void sex_reproduce ()
{
  int i,j;
  popsize = 2 * popsize;

  for (i=0; i< (popsize/2); i++)
  {
      j = 2 * i;
      new_population[j] = new_population[i];
  }

}   /*end of copy */

/********************************************************************
	ASEX_REPRODUCE FUNCTION --Creates 4 asexual individuals
          for every existing sexual individual before mutation
********************************************************************/
void asex_reproduce ()
{
  int i,j,k,m;
  popsize = 4 * popsize;

  for (i=0; i< (popsize/4); i++)
  {
      j = 2 * i;
      k = 3 * i;
      m = 4 * i;

      new_population[j] = new_population[i];
      new_population[k] = new_population[i];
      new_population[m] = new_population[i];
  }

}   /*end of copy */

/****************************************************************
	MUTATION FOR ENVIRONMENT -- mutates the environment for
          the cases (processors) were the environment changes
****************************************************************/
void mutate_environment()
{
  if (randval(0, 100) < MUT_ENVIRONMENT)
      environment.gene[randval(0, 5)] = randval(0, 10);

}	/*end of mutate_environment */
/********************************************************
	CROSSOVER FUNCTION FOR THE SEXUAL REPRODUCTION
            one point crossover
*********************************************************/
void crossover()
{
  int i, j;
  int parent1[NVARS], parent2[NVARS];
  int x_point;
  int mate;			/* second parent to mate with*/

   for (i=0; i<popsize; i++)
   {
      if(randval(0, 100) < X_OVER)
      {
         x_point = randval(1, 5);
         mate = randval(0, popsize);

         for (j = 0; j< NVARS; j++)
         {
            parent1[j] = new_population[i].gene[j];
            parent2[j] = new_population[mate].gene[j];
         }

         for (j=0; j<x_point; j++)
          {
             int temp = parent1[j];
             parent1[j] = parent2[j];
             parent2[j] = temp;
          }
          for (j = 0; j< NVARS; j++)
          {
             new_population[i].gene[j] = parent1[j];
             new_population[mate].gene[j] = parent2[j];
          }
       }	/*end of if */
    }	/* end of for i */
}	/*end of crossover */

void print()  /* a function for printing results*/
{
  int count;
  int i, j;
  int allZeros;
  float report_fitness;
  /* print the number of individuals for each population and
  print the average fitness for the population  */

  count = 0;
  for (i=0; i<popsize; i++)  /* get num of viable individuals*/
  {
    allZeros = TRUE;
    for(j = 0; j < NVARS; j++)
    {
      if (population[i].gene[j] != 0)
        allZeros = FALSE;
    }
    if(allZeros == FALSE)
      count += 1;
  } /* end for*/

  report_fitness = (10.0 - avg_pop_fit)/10.0;
  printf("Population Size is: %d\n", count);
  printf("The average fitness for this population is %f\n",
             report_fitness);

}	/* end of print function */
/**********************************************
		MAIN FUNCTION
**********************************************/
void main(int argc, char*argv[])
{
  int h, i, j, k, rank, num_procs, dest;
  MPI_Status status;
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &num_procs);

  if (rank == 0)
  {
    printf("generations:  %d\n", GENERATIONS);
    printf("genes per indiv.:  %d\n", NVARS);
    printf("mutation rate:  %f\n", (float) MUT_IND/100.0);
    printf("crossover rate:  %f\n", (float) X_OVER/100.0);
    printf("environ. mutation rate: %f\n", (float) MUT_ENVIRONMENT/100.0);
    printf("starting population size: %d\n", START_POP);
  } /*end if*/

  init(&environment);  /*initialize the environment*/
  for(i = 0; i < popsize; i++)
  {
   init(&(population[i]));  /*initialize populations*/
   new_population[i] = population[i];
  } /* end for*/
  evaluate();  /*evaluates population against the environment*/

  /* This is where mutation and crossover occur, and mutation
     of the environment.  Process 0 contains an asexually reproducing
     population in a static environment.  Process 1 holds an asexually
     reproducing population in a changing environment.  Process 2 holds
     a sexually reproducing population in a static environment.  Finally
     Process 3 holds a sexually reproducing population in a changing
     environment */

  for(i=0; i<GENERATIONS; i++)
  {
     if (rank == 1 || rank == 3)
        mutate_environment();    /*only call in changing environments*/
     if(rank < 2)
     {
       if(popsize < 250)
          asex_reproduce(); /*call for asexual populations*/
     }
     else
     {
       if(popsize < 500)
          sex_reproduce();  /*call for sexual populations*/
     }
     mutate();
     if(rank == 2 || rank ==3)
       crossover();   /*only call for sexually reproducing*/
                      /*populations*/
     for(j=0; j<popsize; j++)
       for (k=0; k<NVARS; k++)
         population[j].gene[k] = new_population[j].gene[k];
     evaluate();

     if(i==0  || i%10==0)
     {
       printf("rank: %d\n", rank);
       print();  /*print out fitness and popsize every 10 generations*/
     }	/*end of if i =0, i%10 */

  }

  if (rank == 0)
    printf("Asexual population in a static environment");
  if (rank == 1)
    printf("Asexual population in a changing environment");
  if (rank == 2)
    printf("Sexual population in a static environment");
  if (rank == 3)
    printf("Sexual population in a changing environment");
  printf("\n");
  print();
  printf("\n");
  MPI_Finalize();

}  /*end main*/

