!This program was originally written by David Walker. I modified its program

        program sor
        implicit none
        include 'mpif.h'
        integer myrank, ierr, nprocs, nprocx, nprocy, pseudohost
        integer nptx, npty
        integer MAXX, MAXY
        parameter (pseudohost = 0)
        parameter (MAXX = 100, MAXY = 100)
        real phi(0:MAXY+1,0:MAXX+1)
        double precision phi_old(0:MAXY+1,0:MAXX+1),phi_new(0:MAXY+1,0:MAXX+1)
        logical mask(0:MAXY+1,0:MAXX+1)
        integer color(0:MAXY+1,0:MAXX+1)
        integer i, j, k, ix, iy, ipx, ipy, jpx, jpy
        integer RED, BLACK
        parameter (RED = 0, BLACK = 1)
        double precision timebegin, timeend
        integer insize, pos, src, m, n, status(MPI_STATUS_SIZE)
        double precision omega, omega4, omegam, buf(MAXY)
        integer ldest, rdest, udest, ddest
        integer lsrc, rsrc, usrc, dsrc
        integer dims(2), coords(2), from(2)
        logical periods(2)
        logical reorder
        integer comm2d
        integer typex, typey
        double precision  mymax,overmax
        double precision  convg 
        parameter (convg= 0.003)
        integer up,down,right,left
        integer maxiter,iter
        parameter (maxiter = 1000)



!
!  Initialize MPI, find rank of each process, and the number of processes
!
        
        call mpi_init (ierr)
        call mpi_comm_rank (MPI_COMM_WORLD, myrank, ierr)
        call mpi_comm_size (MPI_COMM_WORLD, nprocs, ierr)
        omega = 0.25


!  One process acts as the host and reads in the input
!

        nprocx = 2
        nprocy = 4
        nptx   = MAXX
        npty   = MAXY
      !
!  Abort if incorrect.
!
        if (nprocx .lt. 0 .or. nprocy .lt. 0) then
           print *, 'Number of processes in each direction must be >0'
           goto 999
        else if (nprocx*nprocy .ne. nprocs) then
           print *, 'nprocx*nprocy must = ',nprocs
           goto 999
        endif

        if (nptx .gt. MAXX) then
           print *, 'Number of points per process must be <= ',MAXX
           goto 999
        end if
        if (npty .gt. MAXY) then
           print *, 'Number of points per process must be <= ',MAXY
           goto 999
        end if

        if (mod(nptx,2) .ne. 0 .or. mod(npty,2) .ne. 0) then
           print *, 'Number of points per process must be even'
           goto 999
        end if

        timebegin = mpi_wtime ()

!  Set up 2D application topology

        periods(1) = .false.
        periods(2) = .false.
        dims(1)    = nprocy
        dims(2)    = nprocx
        reorder    = .false.
        call mpi_cart_create (MPI_COMM_WORLD, 2, dims, periods, reorder,&
                             comm2d, ierr)
        call mpi_cart_coords (comm2d, myrank, 2, coords, ierr)
        call mpi_cart_shift (comm2d, 0,  1, dsrc, ddest, ierr)
        call mpi_cart_shift (comm2d, 0, -1, usrc, udest, ierr)
        call mpi_cart_shift (comm2d, 1,  1, rsrc, rdest, ierr)
        call mpi_cart_shift (comm2d, 1, -1, lsrc, ldest, ierr)

!  Set up the grid mask and grid arrays

        call setup_grid (phi_old, color, nptx, npty, &
                         coords, nprocx, nprocy,&
                        mask, MAXY+1)
!        print *,'coords',coords,'me',myrank, 'neigh',ddest, &
!          udest,rdest,ldest,phi_old(1:npty,1:nptx)
!        stop

        up = 1
        down = npty
        left = 1
        right = nptx
        if (mask(1,2) .eqv. .false.) up = 2
        if (mask(npty,2) .eqv. .false.) down = npty-1
        if (mask(2,1) .eqv. .false.) left = 2
        if (mask(2,nptx) .eqv. .false.) right = nptx-1          



!  Set up general datatypes

        call mpi_type_contiguous (npty, mpi_double_precision, typex, ierr)
        call mpi_type_vector (nptx, 1, npty+2, mpi_double_precision, typey, ierr) 
        call mpi_type_commit (typex, ierr)
        call mpi_type_commit (typey, ierr)

!  Start main loop

        omega4 = 0.25*omega
        omegam = 1.0 - omega

        k = 1
        do iter=1,maxiter 

!  Exchange black boundary data

           call mpi_sendrecv (phi_old(1,1), 1, typex, ldest,  14, & 
                             phi_old(1,nptx+1), 1, typex, lsrc,   14, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_old(1,nptx), 1, typex, rdest, 15, & 
                             phi_old(1,0),    1, typex, rsrc,  15, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_old(1,1),      1, typey, udest, 16, & 
                             phi_old(npty+1,1), 1, typey, usrc,  16, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_old(npty,1), 1, typey, ddest, 17, & 
                             phi_old(0,1),    1, typey, dsrc,  17, & 
                             comm2d, status, ierr)

      
!  Update red points

           do j=1,nptx
              do i=1,npty
                 if (mask(i,j) .and. color(i,j) .eq. RED) phi_new(i,j) =  &
                   omega4*(phi_old(i,j-1)+phi_old(i-1,j)+phi_old(i,j+1)+ &
                    phi_old(i+1,j)) + omegam*phi_old(i,j)
              end do
           end do

           call mpi_sendrecv (phi_new(1,1), 1, typex, ldest,  10, & 
                             phi_new(1,nptx+1), 1, typex, lsrc,   10, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_new(1,nptx), 1, typex, rdest, 11, & 
                             phi_new(1,0),    1, typex, rsrc,  11, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_new(1,1),      1, typey, udest, 12, & 
                             phi_new(npty+1,1), 1, typey, usrc,  12, & 
                             comm2d, status, ierr)
           call mpi_sendrecv (phi_new(npty,1), 1, typey, ddest, 13, & 
                             phi_new(0,1),    1, typey, dsrc,  13, & 
                             comm2d, status, ierr)

!            print *,'coords',coords,'me',myrank, 'neigh',ddest, &
!           udest,rdest,ldest,phi_new(1:npty,1:nptx)
!           stop


!  Update black points
!
           do j=1,nptx
              do i=1,npty
                 if (mask(i,j) .and. color(i,j) .eq. BLACK) phi_new(i,j) = &
                   omega4*(phi_new(i,j-1)+phi_new(i-1,j)+phi_new(i,j+1)+ &
                  phi_new(i+1,j))  + omegam*phi_old(i,j)
              end do
           end do
          
           phi_old(up:down,left:right) = phi_new(up:down,left:right)
        end do

!  End main loop

        call mpi_type_free (typex, ierr)
        call mpi_type_free (typey, ierr)

!  Stop clock and write out timings

        timeend = mpi_wtime()
        print *,'Node', myrank,'  Elapsed time: ',& 
               timeend-timebegin,' seconds'

!  Print out results

        call mpi_comm_free (comm2d, ierr)
  999   call mpi_finalize (ierr)

        stop
        end

        subroutine setup_grid (phi, color, nptx, npty, &  
                               coords, nprocx, nprocy, & 
                              mask, ldim)
        implicit none
        integer ldim
        double precision  phi(0:ldim,0:*)
        integer color(0:ldim,0:*), nptx, npty
        integer  coords(2), nprocx, nprocy
        logical mask(0:ldim,0:*)
        integer i, j, ix, iy, ipx, ipy, jpx, jpy 
        integer RED, BLACK
        parameter (RED = 0, BLACK = 1)
        integer x,y       

        do j=1,nptx
           do i=1,npty
              mask(i,j)    = .true.
              phi(i,j)     = 0.0d0
              if (mod(i+coords(1)*npty+j+coords(2)*nptx,2) == 0) then
                   color(i,j) = RED
              else
                  color(i,j) = BLACK
              end if
           end do
        end do

        if (coords(2) .eq. 0) then
           do i=1,npty
              mask(i,1) = .false.
              y = 1 
              x = i + coords(1)*npty
              phi(i,1)  = -4.7 + 0.02*x - 0.03*y +0.09* &
                 ((x/10.0)**2-(y/10.0)**2)+ &
                log(  sqrt(((0.1*x+0.2)**2) +((0.1*y+0.2)**2)))
           end do
        end if
        if (coords(2) .eq. nprocx-1) then
           do i=1,npty
              mask(i,nptx) = .false.
              y = nptx + coords(2)*nptx
              x = i + coords(1)*npty
              phi(i,nptx)  = -4.7 + 0.02*x - 0.03*y +0.09* &
                 ((x/10.0)**2-(y/10.0)**2)+ &
                log(  sqrt(((0.1*x+0.2)**2) +((0.1*y+0.2)**2)))
           end do
        end if
        if (coords(1) .eq. 0) then
           do i=1,nptx
              mask(1,i) = .false.
              y = i + coords(2)*nptx
              x = 1 
              phi(1,i)  = -4.7 + 0.02*x - 0.03*y +0.09* &
                 ((x/10.0)**2-(y/10.0)**2)+ &
                log(  sqrt(((0.1*x+0.2)**2) +((0.1*y+0.2)**2)))
           end do
        end if
        if (coords(1) .eq. nprocy-1) then
           do i=1,nptx
              mask(npty,i) = .false.
              y = i + coords(2)*nptx 
              x = npty + coords(1)*npty
              phi(npty,i)  = -4.7 + 0.02*x - 0.03*y +0.09* &
                 ((x/10.0)**2-(y/10.0)**2)+ &
                log(  sqrt(((0.1*x+0.2)**2) +((0.1*y+0.2)**2)))
           end do
        end if

      
        return
        end









