/*
 * File         : MPI.java
 * Author       : Sang Lim, Sung-Hoon Ko, Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1999/03/03 19:55:39 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

package mpi;
//import mpi.*;

public class MPI {

  static int MAX_PROCESSOR_NAME = 256;

  static public Intracomm COMM_WORLD;
  static public Comm COMM_SELF;

  static public int GRAPH, CART;
  static public int ANY_SOURCE, ANY_TAG;

  static public Op MAX, MIN, SUM, PROD, LAND, BAND,
    LOR, BOR, LXOR, BXOR, MINLOC, MAXLOC;
  static public Datatype UNDEFINED, BYTE, CHAR, SHORT, BOOLEAN, 
    INT, LONG, FLOAT, DOUBLE, PACKED, LB, UB, OBJECT;
  static public Datatype SHORT2, INT2, LONG2, FLOAT2, DOUBLE2;

  static public Request REQUEST_NULL;
  static public Group GROUP_EMPTY;

  static public int PROC_NULL;
  static public int BSEND_OVERHEAD;
//  static public int UNDEFINED;
  static public int IDENT, CONGRUENT, SIMILAR, UNEQUAL;
  static public int TAG_UB, HOST, IO;

  static Errhandler ERRORS_ARE_FATAL, ERRORS_RETURN;

  static private native void InitNative(String[] args);
  static private native void SetConstant();

  /**
   * Initialize MPI.
   * <p>
   * <table>
   * <tr><td><tt> args </tt></td><td> arguments to <tt>main</tt> method. </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_INIT</tt>.
   */

  static public void Init(String[] args) {
    InitNative(args);
/*
  private final static int MAX   = 1;
  private final static int MIN   = 2;
  private final static int SUM   = 3;
  private final static int PROD  = 4;
  private final static int LAND  = 5;
  private final static int BAND  = 6;
  private final static int LOR   = 7;
  private final static int BOR   = 8;
  private final static int LXOR  = 9;
  private final static int BXOR  =10;
  private final static int MINLOC=11;
  private final static int MAXLOC=12;
*/
/*
    MAX = new Op(Op.MAX);
    MIN = new Op(Op.MIN);
    SUM = new Op(Op.SUM);
    PROD = new Op(Op.PROD);
    LAND = new Op(Op.LAND);
    BAND = new Op(Op.BAND);
    LOR = new Op(Op.LOR);
    BOR = new Op(Op.BOR);
    LXOR = new Op(Op.LXOR);
    BXOR = new Op(Op.BXOR);
    MINLOC = new Op(Op.MINLOC);
    MAXLOC = new Op(Op.MAXLOC);
*/
    MAX = new Op(1);
    MIN = new Op(2);
    SUM = new Op(3);
    PROD = new Op(4);
    LAND = new Op(5);
    BAND = new Op(6);
    LOR = new Op(7);
    BOR = new Op(8);
    LXOR = new Op(9);
    BXOR = new Op(10);
    MINLOC = new Op(11);
    MAXLOC = new Op(12);

/*
  private final static int BYTE    = 1;
  private final static int CHAR    = 2;
  private final static int SHORT   = 3;
  private final static int BOOLEAN = 4;
  private final static int INT     = 5;
  private final static int LONG    = 6;
  private final static int FLOAT   = 7;
  private final static int DOUBLE  = 8;  
  private final static int PACKED  = 9;
  private final static int LB      =10;
  private final static int UB      =11;
  private final static int OBJECT  = 12;
*/
/*
    BYTE = new Datatype(Datatype.BYTE);
    CHAR = new Datatype(Datatype.CHAR);
    SHORT = new Datatype(Datatype.SHORT);
    BOOLEAN = new Datatype(Datatype.BOOLEAN);
    INT = new Datatype(Datatype.INT);
    LONG = new Datatype(Datatype.LONG);
    FLOAT = new Datatype(Datatype.FLOAT);
    DOUBLE = new Datatype(Datatype.DOUBLE);
    PACKED = new Datatype(Datatype.PACKED);
    LB = new Datatype(Datatype.LB);
    UB = new Datatype(Datatype.UB);
    OBJECT = new Datatype(Datatype.OBJECT);
*/

    BYTE = new Datatype(1);
    CHAR = new Datatype(2);
    SHORT = new Datatype(3);
    BOOLEAN = new Datatype(4);
    INT = new Datatype(5);
    LONG = new Datatype(6);
    FLOAT = new Datatype(7);
    DOUBLE = new Datatype(8);
    PACKED = new Datatype(9);
    LB = new Datatype(10);
    UB = new Datatype(11);
    OBJECT = new Datatype(12);

    GROUP_EMPTY = new Group(Group.EMPTY);
    REQUEST_NULL = new Request(Request.NULL);
    
    // Constant
    SetConstant(); 

    ERRORS_ARE_FATAL= new Errhandler(Errhandler.FATAL);
    ERRORS_RETURN = new Errhandler(Errhandler.RETURN);       

    COMM_WORLD = new Intracomm(Intracomm.WORLD);
  }

  /**
   * Finalized MPI.
   * <p>
   * Java binding of the MPI operation <tt>MPI_FINALIZE</tt>.
   */

  static public native void Finalize();

  /**
   * Returns wallclock time.
   * <p>
   * <table>
   * <tr><td><em> returns: </em></td><td> elapsed wallclock time in seconds
   *                                      since some time in the past </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_WTIME</tt>.
   */

  static public native double Wtime();

  /**
   * Returns resolution of timer.
   * <p>
   * <table>
   * <tr><td><em> returns: </em></td><td> resolution of <tt>wtime</tt> in
   *                                      seconds. </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_WTICK</tt>.
   */

  static public native double Wtick();

  /**
   * Returns the name of the processor on which it is called.
   * <p>
   * <table>
   * <tr><td><em> returns: </em></td><td> A unique specifier for the actual
   *                                      node. </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_GET_PROCESSOR_NAME</tt>.
   */

  static public String Get_processor_name() {
    byte[] buf = new byte[MAX_PROCESSOR_NAME] ;
    int lengh = Get_processor_name(buf) ;
    return new String(buf,0,lengh) ;
  }

  static private native int Get_processor_name(byte[] buf) ;

  /**
   * Test if MPI has been initialized.
   * <p>
   * <table>
   * <tr><td><em> returns: </em></td><td> <tt>true</tt> if {\tt init} has
   *                                      been called, <tt>false</tt>
   *                                      otherwise. </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_INITIALIZED</tt>.
   */

  static public native boolean Initialized();

  // Buffer allocation

  /**
   * Provides to MPI a buffer in user's memory to be used for buffering
   * outgoing messages.
   * Java binding of the MPI operation <tt>MPI_BUFFER_ATTACH</tt>.
   */

  static public native void Buffer_attach(byte[] buffer);

  /**
   * Detach the buffer currently associated with MPI.
   * Java binding of the MPI operation <tt>MPI_BUFFER_DETACH</tt>.
   */

  static public native void Buffer_detach(byte[] buf);
   
  static {
    System.loadLibrary("mpijava");
  }
}

// Things to do:
//
//   Correct interface to `Buffer_detach'.

