/*
 * File         : Prequest.java
 * Author       : Sang Lim, Xinying Li, Bryan Carpenter
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1999/03/03 19:55:42 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

package mpi;
//import mpi.*;

public class Prequest extends Request {

  /**
   * Activate a persistent communication request.
   * Java binding of the MPI operation <tt>MPI_START</tt>.
   * The communication is completed by using the request in
   * one of the <tt>wait</tt> or <tt>test</tt> operations.
   * On successful completion the request becomes inactive again.
   * It can be reactivated by a further call to <tt>Start</tt>.
   */

  private native void start();
  public void Start(){
    switch (opTag) {
      case Tag_Normal :
	  start();
	break;
      case Tag_Send_Object :
          comm.Isend(length_buf,0, 2, MPI.INT, dest, tag);
          dataReq = comm.Isend(byte_buf, 0,    length_buf[0], 
                               MPI.BYTE, dest, tag);
        break;
      case Tag_Bsend_Object :
           comm.Ibsend(length_buf,0, 2, MPI.INT, dest, tag);
           dataReq = comm.Ibsend(byte_buf, 0,    length_buf[0], 
                                 MPI.BYTE, dest, tag);
         break;
       case Tag_Ssend_Object :
           comm.Isend(length_buf,0, 2, MPI.INT, dest, tag);
           dataReq = comm.Isend(byte_buf, 0,    length_buf[0], 
                                MPI.BYTE, dest, tag);
         break;
       case Tag_Rsend_Object :
           comm.Irsend(length_buf,0, 2, MPI.INT, dest, tag);
           dataReq = comm.Irsend(byte_buf, 0,    length_buf[0], 
                                 MPI.BYTE, dest, tag);
         break;
       case Tag_Recv_Object :
         break;
       default : 
           System.out.println("Unknown Tag..."+opTag);
         break;
    }   	   
  }

  /**
   * Activate a list of communication requests.
   * <p>
   * <table>
   * <tr><td><tt> array_of_requests </tt></td><td> array of requests </tr>
   * </table>
   * <p>
   * Java binding of the MPI operation <tt>MPI_STARTALL</tt>.
   */

  private static native void startall(Request [] array_of_request);
  public static void Startall(Request [] array_of_request){
    if (array_of_request[0].opTag == Tag_Normal)
      startall(array_of_request);
    else {
      int req_length = array_of_request.length;
      for (int i = 0; i<req_length; i++){
        switch (array_of_request[i].opTag){
          case Tag_Send_Object :
              array_of_request[i].comm.Isend(array_of_request[i].length_buf,
                                             0, 2, MPI.INT,
                                             array_of_request[i].dest,
                                             array_of_request[i].tag);

              array_of_request[i].dataReq = 
                    array_of_request[i].comm.Isend(
                          array_of_request[i].byte_buf, 0, 
                          array_of_request[i].length_buf[0], MPI.BYTE, 
                          array_of_request[i]. dest,array_of_request[i]. tag);
            break;
          case Tag_Bsend_Object :
              array_of_request[i].comm.Ibsend(array_of_request[i].length_buf,
                                              0, 2, MPI.INT,
                                              array_of_request[i].dest,
                                              array_of_request[i].tag);

              array_of_request[i].dataReq = array_of_request[i].comm.Ibsend(
                          array_of_request[i].byte_buf, 0, 
                          array_of_request[i].length_buf[0], MPI.BYTE, 
                          array_of_request[i].dest, array_of_request[i].tag);
            break;
          case Tag_Ssend_Object :
              array_of_request[i].comm.Isend(array_of_request[i].length_buf,
                                             0, 2,  MPI.INT, 
                                             array_of_request[i].dest, 
                                             array_of_request[i].tag);
              array_of_request[i].dataReq = 
                    array_of_request[i].comm.Isend(
                          array_of_request[i].byte_buf, 0, 
                          array_of_request[i].length_buf[0], MPI.BYTE, 
                          array_of_request[i].dest, array_of_request[i].tag);
            break;
          case Tag_Rsend_Object :
              array_of_request[i].comm.Irsend(array_of_request[i].length_buf,
                                              0, 2, MPI.INT, 
                                              array_of_request[i].dest, 
                                              array_of_request[i].tag);
              array_of_request[i].dataReq = 
                    array_of_request[i].comm.Irsend(
                          array_of_request[i].byte_buf, 0, 
                          array_of_request[i].length_buf[0], MPI.BYTE, 
                          array_of_request[i].dest, array_of_request[i].tag);
            break;
          case  Tag_Recv_Object :
            break;
          default : 
              System.out.println("Unknown Tag..."+array_of_request[i].opTag);
            break;
        }
      }   	
    }
  }
}

