/*
 MPI-Java version :
    Sang Lim (slim@npac.syr.edu)
    Northeast Parallel Architectures Center at Syracuse University
    12/2/98
*/

import mpi.*;

class alltoallv {
  static public void main(String[] args) {
    
    final int MAXLEN = 10;

    int myself,tasks;
    MPI.Init(args);
    myself = MPI.COMM_WORLD.Rank();
    tasks = MPI.COMM_WORLD.Size();

    int root,i=0,j,k,stride=15;
    int out[]    = new int[tasks*stride];
    int in[]     = new int[tasks*stride];
    int sdis[]   = new int[tasks];
    int scount[] = new int[tasks];
    int rdis[]   = new int[tasks];
    int rcount[] = new int[tasks];

    
    for(i = 0; i < tasks;i++){
       sdis[i] = i*stride;
       scount[i] = 15;
       rdis[i] = i*15;
       rcount[i] = 15;
    }

    if(myself == 0)
       for(i = 0; i < tasks;i++)
          scount[i] = 10;
    
    rcount[0] = 10;
    for(j=0;j<tasks;j++)
       for (i = 0;i < stride;i++){
          out[i+j*stride] = i + myself * stride;
          in[i+j*stride] = 0;
       }

    MPI.COMM_WORLD.Alltoallv(out,0,scount,sdis,MPI.INT,in,0,rcount,rdis,MPI.INT);
    if(myself==0){
       System.out.println("Original array on Proc 0...");
       for(i=0; i<tasks*stride; i++)
          System.out.print(out[i]+" ");
       System.out.println();
       System.out.println();
    }
    MPI.COMM_WORLD.Barrier();

    if(myself==1){
       System.out.println("Original array on Proc 1...");
       for(i=0; i<tasks*stride; i++)
          System.out.print(out[i]+" ");
       System.out.println();
       System.out.println();
    }
    MPI.COMM_WORLD.Barrier();

    if(myself==2){
       System.out.println("Original array on Proc 2...");
       for(i=0; i<tasks*stride; i++)
          System.out.print(out[i]+" ");
       System.out.println();
       System.out.println();
    }
    MPI.COMM_WORLD.Barrier();

    if(myself==0){
       System.out.println("Result on proc 0...");
       System.out.println("Send Stride = 15 "+"Recv Displacement = " 
                          +rdis[0]+","+rdis[1]+","+rdis[2]+
                          " Count = "+scount[0]);
       for(i=0; i<tasks*stride; i++)
          System.out.print(in[i]+" ");
       System.out.println();
       System.out.println();
    }
    
    MPI.COMM_WORLD.Barrier();

    if(myself==1){
       System.out.println("Result on proc 1...");
       System.out.println("Send Stride = 15 "+"Recv Displacement = " 
                          +rdis[0]+","+rdis[1]+","+rdis[2]+
                          " Count = "+scount[0]);
       for(i=0; i<tasks*stride; i++)
          System.out.print(in[i]+" ");
       System.out.println();
       System.out.println();
    }
    MPI.COMM_WORLD.Barrier();

    if(myself==2){
       System.out.println("Result on proc 2...");
       System.out.println("Send Stride = 15 "+"Recv Displacement = " 
                          +rdis[0]+","+rdis[1]+","+rdis[2]+
                          " Count = "+scount[0]);
       for(i=0; i<tasks*stride; i++)
          System.out.print(in[i]+" ");
       System.out.println();
    }
    MPI.COMM_WORLD.Barrier();
    if(myself == 0)  System.out.println("Alltoallv TEST COMPLETE\n");
    MPI.Finalize();
  }
}
