/*
 MPI-Java version :
    Sang Lim (slim@npac.syr.edu)
    Northeast Parallel Architectures Center at Syracuse University
    12/3/98
*/

import mpi.*;

class alltoallvO {
  static public void main(String[] args) {
    
    final int MAXLEN = 10;

    int myself,tasks;
    MPI.Init(args);
    myself = MPI.COMM_WORLD.Rank();
    tasks = MPI.COMM_WORLD.Size();

    int root,i=0,j,k,stride=15;
    int out[][]    = new int[tasks*stride][MAXLEN];
    int in[][]     = new int[tasks*stride][MAXLEN];
    int sdis[]   = new int[tasks];
    int scount[] = new int[tasks];
    int rdis[]   = new int[tasks];
    int rcount[] = new int[tasks];

    
    for(i = 0; i < tasks;i++){
       sdis[i] = i*stride;
       scount[i] = 15;
       rdis[i] = i*15;
       rcount[i] = 15;
    }

    if(myself == 0)
       for(i = 0; i < tasks;i++)
          scount[i] = 10;
    
    rcount[0] = 10;

    for(j=0;j<tasks;j++)
      for (i = 0;i < stride;i++)
        for(k = 0; k< MAXLEN; k++){
          out[i+j*stride][k] = i + myself * stride;
          in[i+j*stride][k] = 0;
        }
       

    MPI.COMM_WORLD.Alltoallv(out, 0, scount, sdis, MPI.OBJECT,
                             in , 0, rcount, rdis, MPI.OBJECT);

    for(j = 0; j < tasks; j++)
      if(myself==j){
        System.out.println("Original array on Proc "+j+"...");
        for(k=0; k<MAXLEN; k++){
          for(i=0; i<tasks*stride; i++)
            System.out.print(out[i][k]+" ");
          System.out.println();
        }

        System.out.println();
        System.out.println();
      }
    MPI.COMM_WORLD.Barrier();

    for(j = 0; j < tasks; j++){
      if(myself==j){
        System.out.println("Result on proc "+j+"...");
        System.out.println("Send Stride = 15 "+"Recv Displacement = " 
                           +rdis[0]+","+rdis[1]+
                           " Count = "+scount[0]);

        for(k=0; k<MAXLEN; k++){
          for(i=0; i<tasks*stride; i++)
            System.out.print(in[i][k]+" ");
          System.out.println();
        }

        System.out.println();
        System.out.println();
      }
      MPI.COMM_WORLD.Barrier();
    }

    MPI.COMM_WORLD.Barrier();
    if(myself == 0)  System.out.println("AlltoallvO TEST COMPLETE\n");
    MPI.Finalize();
  }
}
