! N-BODY with parallel Random number generator
! This program was modified from the original in the part of
! random particles (mass, coordinate)
! The original code make calls to 'random_number' which is not a
! parallel RNG.
! In this modified code, I used the 'EXTRINSIC' subroutine to
! generate random numbers in parallel.
! Moreover, the initial 'SEEDS' is a sequence of random numbers which
! is generated randomly with the 'SEED' from 'DATE_AND_TIME'. This
! is to make the 'uncorrelated seeds' for processors.

! PROGRAM by KAYASIDH KASYAPANUN
! CPS615 FALL 1996 , HOMEWORK # 8
! November 7, 1996

	PROGRAM NBODY_parallelRNG

        INTERFACE
	  EXTRINSIC(HPF_LOCAL) SUBROUTINE get_random(x,seed_v)
	  REAL, DIMENSION(:) :: x
          INTEGER, DIMENSION(:) :: seed_v
!HPF$ DISTRIBUTE *(BLOCK) :: x
!HPF$ DISTRIBUTE *(BLOCK) :: seed_v
          END SUBROUTINE get_random
        END INTERFACE

        INTEGER, PARAMETER :: NN = 1024
        INTEGER, PARAMETER :: MM=0, PX=1, PY=2, PZ=3, FX=4, FY=5, FZ=6

!       MM:mass, PX:position in x coordinate, PY:position in y coordinate
!       FX: force in x coordinate, FY: force in y coordinate

        REAL, DIMENSION(0:NN-1) :: x,dx,dy,dz,tx,ty,tz,sq,dist,fac
        REAL, DIMENSION(0:NN-1,0:6) :: p,q
        INTEGER :: i,k,npts
        INTEGER :: t1,t2,t3,rate,num_proc,vec_size,seed_size
        INTEGER, ALLOCATABLE :: seed_vec(:)
        REAL, ALLOCATABLE :: seed_init(:)
        INTEGER :: data_time(8)

!HPF$ DISTRIBUTE x(BLOCK)
!HPF$ DISTRIBUTE seed_vec(BLOCK)	! distributed seeds
!HPF$ ALIGN (:) WITH x(:) :: dx,dy,dz,tx,ty,tz,sq,dist,fac
!HPF$ ALIGN (:,*) WITH x(:) :: p,q

        npts = NN       ! equal to array size (in parameter)

        num_proc = NUMBER_OF_PROCESSORS()
        seed_size = 64 / BIT_SIZE(0)	! no. of integers to hold a seed
        vec_size = seed_size * num_proc	! arr size to hold the seeds
        ALLOCATE (seed_vec (vec_size))	! for hold real seeds
	ALLOCATE (seed_init (vec_size))	! for uncorrelated seeds

        CALL DATE_AND_TIME(values=data_time)	! get a seed from time
        CALL RANDOM_SEED(SIZE = seed_size)
        CALL RANDOM_SEED(PUT = (/data_time(8)/))   ! the seed from time
        CALL RANDOM_NUMBER(seed_init)	! initial uncorrelated seed
        seed_vec = (int(seed_init * (2**16 - 1)))     ! change to int

!       Start Timer
        CALL SYSTEM_CLOCK(t1)

! THE FOLLOWING CALLS TO get_random IS A PARALLEL RNG
        CALL get_random(p(:,MM),seed_vec)
!       p(:,MM) = x(:)
        CALL get_random(p(:,PX),seed_vec)
!       p(:,PX) = x(:)
        CALL get_random(p(:,PY),seed_vec)
!       p(:,PY) = x(:)
        CALL get_random(p(:,PZ),seed_vec)
!       p(:,PZ) = x(:)

        CALL SYSTEM_CLOCK(t2)
! THE COMPUTATION PROCESS IS LEAVED UNTOUCH
        p(:,FX) = 0.0
        p(:,FY) = 0.0
        p(:,FZ) = 0.0

        q = p
        k = npts/2 - 1
        do i = 0, k - 1
          q = CSHIFT(q,DIM=1,SHIFT=1)
          dx(0:npts-1) = p(0:npts-1,PX) - q(0:npts-1,PX)
          dy(0:npts-1) = p(0:npts-1,PY) - q(0:npts-1,PY)
          dz(0:npts-1) = p(0:npts-1,PZ) - q(0:npts-1,PZ)
          sq(0:npts-1) = dx(0:npts-1)**2+dy(0:npts-1)**2+dz(0:npts-1)**2
          dist(0:npts-1) = SQRT(sq(0:npts-1))
          fac(0:npts-1) = p(0:npts-1,MM) * q(0:npts-1,MM) / &
        &               (dist(0:npts-1) * sq(0:npts-1))
          tx(0:npts-1) = fac(0:npts-1) * dx(0:npts-1)
          ty(0:npts-1) = fac(0:npts-1) * dy(0:npts-1)
          tz(0:npts-1) = fac(0:npts-1) * dz(0:npts-1)
          p(0:npts-1,FX) = p(0:npts-1,FX)-tx(0:npts-1)
          q(0:npts-1,FX) = q(0:npts-1,FX)+tx(0:npts-1)
          p(0:npts-1,FY) = p(0:npts-1,FY)-ty(0:npts-1)
          q(0:npts-1,FY) = q(0:npts-1,FY)+ty(0:npts-1)
          p(0:npts-1,FZ) = p(0:npts-1,FZ)-tz(0:npts-1)
          q(0:npts-1,FZ) = q(0:npts-1,FZ)+tz(0:npts-1)
        end do

        q = CSHIFT(q,DIM=1,SHIFT=1)
        k = npts/2
        dx(0:k-1) = p(0:k-1,PX) - q(0:k-1,PX)
        dy(0:k-1) = p(0:k-1,PY) - q(0:k-1,PY)
        dz(0:k-1) = p(0:k-1,PZ) - q(0:k-1,PZ)
        sq(0:k-1) = dx(0:k-1)**2+dy(0:k-1)**2+dz(0:k-1)**2
        dist(0:k-1) = SQRT(sq(0:k-1))
        fac(0:k-1) = p(0:k-1,MM) * q(0:k-1,MM) / (dist(0:k-1) * sq(0:k-1))
        tx(0:k-1) = fac(0:k-1) * dx(0:k-1)
        ty(0:k-1) = fac(0:k-1) * dy(0:k-1)
        tz(0:k-1) = fac(0:k-1) * dz(0:k-1)
        p(0:k-1,FX) = p(0:k-1,FX)-tx(0:k-1)
        q(0:k-1,FX) = q(0:k-1,FX)+tx(0:k-1)
        p(0:k-1,FY) = p(0:k-1,FY)-ty(0:k-1)
        q(0:k-1,FY) = q(0:k-1,FY)+ty(0:k-1)
        p(0:k-1,FZ) = p(0:k-1,FZ)-tz(0:k-1)
        q(0:k-1,FZ) = q(0:k-1,FZ)+tz(0:k-1)

        q = CSHIFT(q,DIM=1,SHIFT=npts/2) 
        p(:,FX) = p(:,FX) + q(:,FX)
        p(:,FY) = p(:,FY) + q(:,FY)
        p(:,FZ) = p(:,FZ) + q(:,FZ)

! Stop timer
        CALL SYSTEM_CLOCK(t3,rate)

        print *,'TOTAL TIME : ',(real(t3-t1))/real(rate),'seconds'
        print *,'RNG TIME   : ',(real(t2-t1))/real(rate),'seconds'
        STOP
        END

!=======================================================================
! EXTRINSIC subroutine for parallel RNG, each processor has its own seed
        EXTRINSIC(HPF_LOCAL) SUBROUTINE get_random(x,seed_v)
          REAL, DIMENSION(:) :: x
          INTEGER, DIMENSION(:) :: seed_v
          CALL RANDOM_SEED(PUT = seed_v)
          CALL RANDOM_NUMBER(x)
        END SUBROUTINE get_random

!========================================================================
